require 'hpcloudmangler'
require 'thor'
require 'multi_json'
require 'rest-client'
require 'formatador'
require 'pp'

module HP
  module Cloud
    module Mangler
      class Cli < Thor
        begin
          require 'io/console'
        rescue LoadError
        end
        @options = {}
        @token = ''
        default_task :list_tenants

        desc "list_sec_groups", "Task to list available security groups. Requires a Tenant ID."
        def list_sec_groups
          @auth_opts = HP::Cloud::Mangler::Client.userdata
          @token = authenticate(@auth_opts)
          sg_list_url = generate_uri(@options[:hp_compute_uri], @auth_opts[:auth][:tenantId].to_s, @options[:suffix])
          sg_list = call_rest_uri(sg_list_url, "get", @token[:access][:token][:id])
          data = parse_resp(sg_list)
          Formatador.display_table(data[:security_groups], [:id, :name, :description])
        end
        desc "create_sec_group", "Task to create a new security group"
        def create_sec_group
          auth= HP::Cloud::Mangler::Client.userdata
          @token = authenticate(auth) 
          suffix = "os-security-groups"
          create_sg_url = generate_uri(@options[:hp_compute_uri], auth[:auth][:tenantId].to_s, suffix)
          sg_name = ask("Please provide a security group name (Example: my-sec-group): ")
          sg_desc = ask("Please provide a security group description: ")
          sg_data = { 'security_group' => { 'description' => sg_desc, 'name' => sg_name } }
          begin
            clientconnect = RestClient.post(
              create_sg_url,
              sg_data.to_json,
              rest_header(@token[:access][:token][:id])
              )
          rescue RestClient::ExceptionWithResponse => e
            puts "HTTP failed: #{e.http_code}: #{e.http_body}"
          end
          resp = MultiJson.decode(clientconnect, :symbolize_names => true)
          tabledata = {:security_group => [ resp[:security_group] ] }
          Formatador.display_table(tabledata[:security_group], [:tenant_id, :id, :name, :description])
        end
        desc "delete_sec_group", "Task to delete a security group"
        method_option :groupid, :type => :numeric, :required => true, :desc => "Security Group ID"
        def delete_sec_group
          Process.exit unless options[:groupid]
          suffix = "os-security-groups" + '/' + "#{options[:groupid]}"
          auth= HP::Cloud::Mangler::Client.userdata
          @token = authenticate(auth)
          delete_sg_url = generate_uri(@options[:hp_compute_uri], auth[:auth][:tenantId].to_s, suffix) 
          begin
            resp = RestClient.delete(delete_sg_url, rest_header(@token[:access][:token][:id]))
          rescue RestClient::ExceptionWithResponse => e
            puts "HTTP failed: #{e.http_code}: #{e.http_body}"
          end
          Formatador.display_line('[green]' + "Security group successfully deleted." + " <#{resp.code}>" + '[/]')
        end
        desc "add_sec_grp_to_sec_grp", "*** NOT IMPLEMENTED YET. Task for adding a security group to another security group"
        def add_sec_grp_to_sec_grp

        end
        desc "list_tenants", "Task to list tenants available to a particular user"
        def list_tenants
          auth = HP::Cloud::Mangler::Client.userdata
          @token = authenticate(auth)
          tenants_list_url = generate_uri(@options[:hp_auth_uri], "tenants")
          tenants_list = call_rest_uri(tenants_list_url, "get", @token[:access][:token][:id])
          data = parse_resp(tenants_list)
          pp tenants_list.code
          Formatador.display_table(data[:tenants], [:id, :name, :description])
        end
        desc "list_sg_details", "Task to list the details for a security group"
        method_option :groupid, :type => :numeric, :required => true, :desc => "Security Group ID"
        def list_sg_details
          Process.exit unless options[:groupid]
          suffix = "os-security-groups" + '/' + "#{options[:groupid]}"
          @auth_opts = HP::Cloud::Mangler::Client.userdata
          @token = authenticate(@auth_opts)
          sg_detail_url = generate_uri(@options[:hp_compute_uri], @auth_opts[:auth][:tenantId].to_s, suffix)
          data = parse_resp(call_rest_uri(sg_detail_url, "get", @token[:access][:token][:id]))
          Formatador.display_line
          Formatador.display_line('[bold]' + "Rules for Security Group: " + data[:security_group][:name] + '[/]')
          Formatador.display_table(data[:security_group][:rules], [:id, :from_port, :to_port, :ip_range])
        end
        desc "getauthtoken", "Task to get an HP Cloud auth token"
        def getauthtoken
          auth = HP::Cloud::Mangler::Client.userdata
          @token = authenticate(auth)
          pp @token[:access][:token][:id]
        end
      private
        def authenticate(opts)
          config_file = File.expand_path("~/pso-ops-tools/fog-utils/data/hpcs.json")
          @options = HP::Cloud::Mangler::Settings.load!( config_file, {:suffix => 'os-security-groups'} )
          auth_uri = generate_uri(@options[:hp_auth_uri], "tokens")
          auth_response = call_rest_uri(auth_uri, "post", opts)
          if auth_response.code == 200
            Formatador.display_line('[green]' + "Authentication successful." + '[/]')
            return parse_resp(auth_response)
          else
            Formatador.display_line('[red]' + "Authentication failed with a " + "#{auth_response.code} error code." +'[/]')
            exit
          end
        end
        def generate_uri (base, *tenant, suffix)
          if tenant[0].nil? || tenant[0] == 0
            return base + suffix
          else
            return base + tenant[0].to_s + '/' + suffix
          end
        end

        def rest_header (*token)
          if token[0].nil? || token[0] == 0 
            return {:content_type => :json, :accept => :json}
          else
            return {:content_type => :json, :accept => :json, 'X-Auth-Token' => token[0]}
          end
        end

        def call_rest_uri (uri, request_type="get", *auth_creds)
          if request_type =~ /get/
            begin
              return RestClient.get(uri, rest_header(auth_creds[0]))
            rescue => e
              e.response
            end
          elsif request_type =~ /post/
            begin 
              return RestClient.post(uri, auth_creds.to_json, rest_header)
            rescue => e
              e.response
            end
          end
        end
        def parse_resp (response)
          MultiJson.decode(response, :symbolize_names => true)
        end
      end
    end
  end
end